%% CustomStaticOptimization
close all; clear all; clc;

import org.opensim.modeling.*;

% Load data into MATLAB arrays.
q = readExp('Kinematics_q.sto');
u = readExp('Kinematics_u.sto');
model = Model('subject01_simbody.osim');
state = model.initSystem();
muscles = model.updMuscles();
nMuscles = muscles.getSize();
nameMuscles = {};
for i=1:nMuscles
    nameMuscles(i) = muscles.get(i-1).getName();
end
    
coordinates = model.getCoordinateSet();
nCoordinates = coordinates.getSize();
nameCoordinates = {};
for i=1:nCoordinates
    nameCoordinates(i) = coordinates.get(i-1).getName();
end

time = q.time;
timeBool = find((0.62<=time) & (time<=1.404));
time = time(timeBool);
frame = length(time);
q = rmfield(q, 'time');
u = rmfield(u, 'time');

q2 = zeros(frame, nCoordinates);
u2 = zeros(frame, nCoordinates);

for i=1:nCoordinates
    q2(:,i) = q.(nameCoordinates{i})(timeBool);
    u2(:,i) = u.(nameCoordinates{i})(timeBool);
end

clear q u



MIF= zeros(1,nMuscles); % muscle maximum isometric force
L  = zeros(frame, nMuscles); % muscle length
FL = zeros(frame, nMuscles); % fiber length
TL = zeros(frame, nMuscles); % tendon length
OFL= zeros(1,nMuscles); % optimal fiber length
TSL= zeros(1,nMuscles); % tendon slack length
S  = zeros(frame, nMuscles); % muscle strength
FF = zeros(frame, nMuscles); % fiber force
AFF= zeros(frame, nMuscles); % active fiber force
PFF= zeros(frame, nMuscles); % passive fiber force
TF = zeros(frame, nMuscles); % tendon force
MA = zeros(frame, nCoordinates, nMuscles); % muscle moment arm

% store muscle parameters
for i=1:frame
	fprintf('Muscle Parameters ... at %f\n', time(i))
    
	Q = Vector(); Q.resize(nCoordinates);
	U = Vector(); U.resize(nCoordinates);
    for j=1:nCoordinates
		Q.set(j-1, q2(i,j));
		U.set(j-1, u2(i,j));
    end
	state.setQ(Q);
	state.setU(U);
    
%     model.realizePosition(state);
%     model.realizeVelocity(state);
    model.realizeDynamics(state);
%     model.equilibrateMuscles(state);

    for j=1:nMuscles
        muscle = Millard2012EquilibriumMuscle.safeDownCast(muscles.get(j-1));
        muscle.set_ignore_activation_dynamics(true); % disable activation dynamics
        muscle.set_ignore_tendon_compliance(true); % disable tendon compliance
%         
%         muscle.computeFiberEquilibrium(state);
%         muscle.computeEquilibrium(state);
%         muscle.computeActuation(state);
        model.equilibrateMuscles(state);

        L(i,j)  = muscle.getLength(state);
        FL(i,j) = muscle.getFiberLength(state);
        TL(i,j) = muscle.getTendonLength(state);
        OFL(j)  = muscle.getOptimalFiberLength();
        TSL(j)  = muscle.getTendonSlackLength();
        TL(i,j) = muscle.getTendonLength(state);
        MIF(j)  = muscle.getMaxIsometricForce();
        FF(i,j) = muscle.getFiberForce(state);
        AFF(i,j)= muscle.getActiveFiberForce(state);
        PFF(i,j)= muscle.getPassiveFiberForce(state);
        TF(i,j) = muscle.getTendonForce(state);

        muscle.setActivation(state, 1);
        S(i,j) = muscle.getFiberForce(state);

%         for k=1:nCoordinates
% 			coordinate = coordinates.get(k-1);
% 			coordinate.setValue(state, q2(i,k))
% 			MA(i,k,j) = muscle.computeMomentArm(state, coordinate);
%         end
    end
end

plot(time, S(:,34))
title(nameMuscles(34))
saveas(gcf, 'MatLab_soleus.jpg')

%%
function struct = readExp(file)
    %Read OpenSim STO and MOT files Or any other format that 
    i = 0;
    while true
        try 
            data = dlmread(file, '\t', i, 0);
            row = i;
            break
        catch
            i = i+1;
        end
    end
    % sprintf('Label row number is: %d', row)

    fid = fopen(file);
    for r=1:(row-1)
        fgetl(fid);
    end
    label = fgetl(fid);
    fclose(fid);
    label = strsplit(label, '\t');

    for i = 1:length(label)
        struct.(char(label(i))) = data(:,i);
    end
end