# %%
import opensim as osim
import numpy as np
import matplotlib.pyplot as plt


def readExp(file, sep='\t', deg=True):
	''' Read OpenSim STO and MOT files 
	or any other format that the headers are separated from labels and data by 'endheader' line.
	can return data and labels, dataFrame, or dict'''
	with open(file, mode='r') as f:
		while True:
			line = f.readline()
			if 'endheader' in line:
				line = f.readline()
				break
		labels = line.split('\n')[0].split(sep)
		a = f.readlines()
	data = np.empty((len(a), len(labels)), dtype=np.float64)
	for ii,i in enumerate(a): # rows
		for jj,j in enumerate(i.split(sep)): # columns
			data[ii,jj] = float(j)
	del a
	if deg==False:
		data[:, 1:] = np.deg2rad(data[:, 1:])
	# return pd.DataFrame(data=data, columns=labels)
	# return data, labels
	data2 = dict()
	for i,ii in enumerate(labels):
		data2[ii] = data[:,i]
	return data2


q = readExp('Kinematics_q.sto', deg=False) # angle in radian
u = readExp('Kinematics_u.sto', deg=False) # velocity in radian/s
model = osim.Model('subject01_simbody.osim')

time = q['time']
timeBool = np.logical_and(0.62<=time, time<=1.404)
time = time[timeBool]
frame = len(time)

del q['time']
del u['time']

for i in list(q.keys()):
	q[i] = q[i][timeBool]
	u[i] = u[i][timeBool]

state = model.initSystem()
muscles = model.updMuscles()
nameMuscles = [i.getName() for i in muscles]
nMuscles = muscles.getSize()
coordinates = model.getCoordinateSet()
nCoordinates = coordinates.getSize()
nameCoordinates = [i.getName() for i in coordinates]

# https://simtk.org/api_docs/opensim/api_docs/classOpenSim_1_1Muscle.html#ac2ddf201cb1a2263f9a1c6baa3d7f314
MIF= np.empty(nMuscles) # muscle maximum isometric force
L  = np.empty((frame, nMuscles)) # muscle length
FL = np.empty((frame, nMuscles)) # fiber length
TL = np.empty((frame, nMuscles)) # tendon length
OFL= np.empty(nMuscles) # optimal fiber length
TSL= np.empty(nMuscles) # tendon slack length
S  = np.empty((frame, nMuscles)) # muscle strength
FF = np.empty((frame, nMuscles)) # fiber force
AFF= np.empty((frame, nMuscles)) # active fiber force
PFF= np.empty((frame, nMuscles)) # passive fiber force
TF = np.empty((frame, nMuscles)) # tendon force
MA = np.empty((frame, nCoordinates, nMuscles)) # muscle moment arm

# store muscle parameters
for i in range(frame):
	print('Muscle Parameters ... at', time[i])
	Q = osim.Vector(); Q.resize(nCoordinates)
	U = osim.Vector(); U.resize(nCoordinates)
	for jj in range(nCoordinates):
		Q.set(jj, q[nameCoordinates[jj]][i])
		U.set(jj, u[nameCoordinates[jj]][i])
	state.setQ(Q)
	state.setU(U)

	model.realizeDynamics(state)

	for j in range(nMuscles):
		muscle = osim.Millard2012EquilibriumMuscle.safeDownCast(muscles.get(j))
		muscle.set_ignore_activation_dynamics(True) # disable activation dynamics
		muscle.set_ignore_tendon_compliance(True) # disable tendon compliance

		model.equilibrateMuscles(state)

		L[i,j]  = muscle.getLength(state)
		FL[i,j] = muscle.getFiberLength(state)
		TL[i,j] = muscle.getTendonLength(state)
		OFL[j]  = muscle.getOptimalFiberLength()
		TSL[j]  = muscle.getTendonSlackLength()
		TL[i,j]   = muscle.getTendonLength(state)
		MIF[j]  = muscle.getMaxIsometricForce()
		FF[i,j] = muscle.getFiberForce(state)
		AFF[i,j]= muscle.getActiveFiberForce(state)
		PFF[i,j]= muscle.getPassiveFiberForce(state)
		TF[i,j] = muscle.getTendonForce(state)

		muscle.setActivation(state, 1)
		S[i,j] = muscle.getFiberForce(state)

		for k in range(nCoordinates):
			coordinate = coordinates.get(k)
			coordinate.setValue(state, q[nameCoordinates[k]][i])
			MA[i,k,j] = muscle.computeMomentArm(state, coordinate)


# plt.plot(MA[:, nameCoordinates.index('knee_angle_r'), nameMuscles.index('rect_fem_r')])
# plt.show(block=False)

plt.plot(time, S[:, nameMuscles.index('soleus_r')])
plt.title('soleus_r')
plt.show(block=False)
plt.savefig('Python_soleus.jpg')